package java.awt;

import javax.accessibility.AccessibleRole;
import sun.awt.AWTAccessor;
import javax.accessibility.AccessibleContext;
import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.util.Enumeration;
import java.awt.event.KeyEvent;
import java.awt.peer.MenuPeer;
import java.util.Vector;
import javax.accessibility.Accessible;

public class Menu extends MenuItem implements MenuContainer, Accessible
{
    Vector<MenuComponent> items;
    boolean tearOff;
    boolean isHelpMenu;
    private static final String base = "menu";
    private static int nameCounter;
    private static final long serialVersionUID = -8809584163345499784L;
    private int menuSerializedDataVersion;
    
    public Menu() throws HeadlessException {
        this("", false);
    }
    
    public Menu(String s) throws HeadlessException {
        this(s, false);
    }
    
    public Menu(String s, boolean tearOff) throws HeadlessException {
        super(s);
        this.items = new Vector();
        this.menuSerializedDataVersion = 1;
        this.tearOff = tearOff;
    }
    
    @Override
    String constructComponentName() {
        synchronized (Menu.class) {
            return "menu" + Menu.nameCounter++;
        }
    }
    
    @Override
    public void addNotify() {
        synchronized (this.getTreeLock()) {
            if (this.peer == null) {
                this.peer = Toolkit.getDefaultToolkit().createMenu(this);
            }
            for (int itemCount = this.getItemCount(), i = 0; i < itemCount; ++i) {
                MenuItem item = this.getItem(i);
                item.parent = this;
                item.addNotify();
            }
        }
    }
    
    @Override
    public void removeNotify() {
        synchronized (this.getTreeLock()) {
            for (int itemCount = this.getItemCount(), i = 0; i < itemCount; ++i) {
                this.getItem(i).removeNotify();
            }
            super.removeNotify();
        }
    }
    
    public boolean isTearOff() {
        return this.tearOff;
    }
    
    public int getItemCount() {
        return this.countItems();
    }
    
    @Deprecated
    public int countItems() {
        return this.countItemsImpl();
    }
    
    final int countItemsImpl() {
        return this.items.size();
    }
    
    public MenuItem getItem(int n) {
        return this.getItemImpl(n);
    }
    
    final MenuItem getItemImpl(int index) {
        return (MenuItem)this.items.elementAt(index);
    }
    
    public MenuItem add(MenuItem obj) {
        synchronized (this.getTreeLock()) {
            if (obj.parent != null) {
                obj.parent.remove(obj);
            }
            this.items.addElement(obj);
            obj.parent = this;
            MenuPeer menuPeer = (MenuPeer)this.peer;
            if (menuPeer != null) {
                obj.addNotify();
                menuPeer.addItem(obj);
            }
            return obj;
        }
    }
    
    public void add(String s) {
        this.add(new MenuItem(s));
    }
    
    public void insert(MenuItem menuItem, int i) {
        synchronized (this.getTreeLock()) {
            if (i < 0) {
                throw new IllegalArgumentException("index less than zero.");
            }
            int itemCount = this.getItemCount();
            Vector<MenuItem> vector = new Vector();
            for (; i < itemCount; ++i) {
                vector.addElement(this.getItem(i));
                this.remove(i);
            }
            this.add(menuItem);
            for (int j = 0; j < vector.size(); ++j) {
                this.add((MenuItem)vector.elementAt(j));
            }
        }
    }
    
    public void insert(String s, int n) {
        this.insert(new MenuItem(s), n);
    }
    
    public void addSeparator() {
        this.add("-");
    }
    
    public void insertSeparator(int i) {
        synchronized (this.getTreeLock()) {
            if (i < 0) {
                throw new IllegalArgumentException("index less than zero.");
            }
            int itemCount = this.getItemCount();
            Vector<MenuItem> vector = new Vector();
            for (; i < itemCount; ++i) {
                vector.addElement(this.getItem(i));
                this.remove(i);
            }
            this.addSeparator();
            for (int j = 0; j < vector.size(); ++j) {
                this.add((MenuItem)vector.elementAt(j));
            }
        }
    }
    
    public void remove(int index) {
        synchronized (this.getTreeLock()) {
            MenuItem item = this.getItem(index);
            this.items.removeElementAt(index);
            MenuPeer menuPeer = (MenuPeer)this.peer;
            if (menuPeer != null) {
                menuPeer.delItem(index);
                item.removeNotify();
                item.parent = null;
            }
        }
    }
    
    @Override
    public void remove(MenuComponent o) {
        synchronized (this.getTreeLock()) {
            int index = this.items.indexOf(o);
            if (index >= 0) {
                this.remove(index);
            }
        }
    }
    
    public void removeAll() {
        synchronized (this.getTreeLock()) {
            for (int i = this.getItemCount() - 1; i >= 0; --i) {
                this.remove(i);
            }
        }
    }
    
    @Override
    boolean handleShortcut(KeyEvent keyEvent) {
        int itemCount = this.getItemCount();
        int i = 0;
        while (i < itemCount) {
            if (this.getItem(i).handleShortcut(keyEvent)) {
                return true;
            }
            ++i;
        }
        return false;
    }
    
    @Override
    MenuItem getShortcutMenuItem(MenuShortcut menuShortcut) {
        int itemCount = this.getItemCount();
        int i = 0;
        while (i < itemCount) {
            MenuItem shortcutMenuItem = this.getItem(i).getShortcutMenuItem(menuShortcut);
            if (shortcutMenuItem != null) {
                return shortcutMenuItem;
            }
            ++i;
        }
        return null;
    }
    
    synchronized Enumeration<MenuShortcut> shortcuts() {
        Vector<Object> vector = new Vector();
        for (int itemCount = this.getItemCount(), i = 0; i < itemCount; ++i) {
            MenuItem item = this.getItem(i);
            if ((item instanceof Menu)) {
                Enumeration<MenuShortcut> shortcuts = ((Menu)item).shortcuts();
                while (shortcuts.hasMoreElements()) {
                    vector.addElement(shortcuts.nextElement());
                }
            }
            else {
                MenuShortcut shortcut = item.getShortcut();
                if (shortcut != null) {
                    vector.addElement(shortcut);
                }
            }
        }
        return vector.elements();
    }
    
    @Override
    void deleteShortcut(MenuShortcut menuShortcut) {
        for (int itemCount = this.getItemCount(), i = 0; i < itemCount; ++i) {
            this.getItem(i).deleteShortcut(menuShortcut);
        }
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        objectOutputStream.defaultWriteObject();
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException, HeadlessException {
        objectInputStream.defaultReadObject();
        for (int i = 0; i < this.items.size(); ++i) {
            ((MenuItem)this.items.elementAt(i)).parent = this;
        }
    }
    
    @Override
    public String paramString() {
        return super.paramString() + (",tearOff=" + this.tearOff + ",isHelpMenu=" + this.isHelpMenu);
    }
    
    private static native void initIDs();
    
    @Override
    public AccessibleContext getAccessibleContext() {
        if (this.accessibleContext == null) {
            this.accessibleContext = new Menu.AccessibleAWTMenu();
        }
        return this.accessibleContext;
    }
    
    @Override
    int getAccessibleChildIndex(MenuComponent o) {
        return this.items.indexOf(o);
    }
    
    static {
        Toolkit.loadLibraries();
        if (!GraphicsEnvironment.isHeadless()) {
            initIDs();
        }
        AWTAccessor.setMenuAccessor(new AWTAccessor.MenuAccessor() {
            @Override
            public Vector<MenuComponent> getItems(Menu menu) {
                return menu.items;
            }
        });
        Menu.nameCounter = 0;
    }
    
    protected class AccessibleAWTMenu extends MenuItem.AccessibleAWTMenuItem
    {
        private static final long serialVersionUID = 5228160894980069094L;
        
        @Override
        public AccessibleRole getAccessibleRole() {
            return AccessibleRole.MENU;
        }
    }
}
